#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2006-12-20 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


def encode(params, validate=False):
	'''Create a shipdata binary message payload to pack into an AIS Msg shipdata.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 5 (field automatically set to "5")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - AISversion(uint): Compliant with what edition.  0 is the first edition.
	  - IMOnumber(uint): vessel identification number (different than mmsi)
	  - callsign(aisstr6): Ship radio call sign
	  - name(aisstr6): Vessel name
	  - shipandcargo(uint): what
	  - dimA(uint): FIX: write a description
	  - dimB(uint): FIX: write a description
	  - dimC(uint): FIX: write a description
	  - dimD(uint): FIX: write a description
	  - fixtype(uint): Method used for positioning
	  - ETA(uint): FIX: break this out
	  - draught(udecimal): Maximum present static draught.  25.5 is 25.5 m or greater
	  - destination(aisstr6): Where is the vessel going
	  - dte(uint): Data terminal ready
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=5),6))
	if 'RepeatIndicator' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['RepeatIndicator']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['UserID']),30))
	if 'AISversion' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['AISversion']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	if 'IMOnumber' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['IMOnumber']),30))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),30))
	if 'callsign' in params:
		bvList.append(aisstring.encode(params['callsign'],42))
	else:
		bvList.append(aisstring.encode('@@@@@@@',42))
	if 'name' in params:
		bvList.append(aisstring.encode(params['name'],120))
	else:
		bvList.append(aisstring.encode('@@@@@@@@@@@@@@@@@@@@',120))
	if 'shipandcargo' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['shipandcargo']),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),8))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimA']),9))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimB']),9))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimC']),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dimD']),6))
	if 'fixtype' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['fixtype']),4))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['ETA']),20))
	if 'draught' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['draught'])*Decimal('10')))),8))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(0)),8))
	if 'destination' in params:
		bvList.append(aisstring.encode(params['destination'],120))
	else:
		bvList.append(aisstring.encode('@@@@@@@@@@@@@@@@@@@@',120))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dte']),1))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),1))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a shipdata message 

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 5 (field automatically set to "5")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - AISversion(uint): Compliant with what edition.  0 is the first edition.
	  - IMOnumber(uint): vessel identification number (different than mmsi)
	  - callsign(aisstr6): Ship radio call sign
	  - name(aisstr6): Vessel name
	  - shipandcargo(uint): what
	  - dimA(uint): FIX: write a description
	  - dimB(uint): FIX: write a description
	  - dimC(uint): FIX: write a description
	  - dimD(uint): FIX: write a description
	  - fixtype(uint): Method used for positioning
	  - ETA(uint): FIX: break this out
	  - draught(udecimal): Maximum present static draught.  25.5 is 25.5 m or greater
	  - destination(aisstr6): Where is the vessel going
	  - dte(uint): Data terminal ready
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['MessageID']=5
	r['RepeatIndicator']=int(bv[6:8])
	r['UserID']=int(bv[8:38])
	r['AISversion']=int(bv[38:40])
	r['IMOnumber']=int(bv[40:70])
	r['callsign']=aisstring.decode(bv[70:112])
	r['name']=aisstring.decode(bv[112:232])
	r['shipandcargo']=int(bv[232:240])
	r['dimA']=int(bv[240:249])
	r['dimB']=int(bv[249:258])
	r['dimC']=int(bv[258:264])
	r['dimD']=int(bv[264:270])
	r['fixtype']=int(bv[270:274])
	r['ETA']=int(bv[274:294])
	r['draught']=Decimal(int(bv[294:302]))/Decimal('10')
	r['destination']=aisstring.decode(bv[302:422])
	r['dte']=int(bv[422:423])
	r['Spare']=0
	return r

def decodeMessageID(bv, validate=False):
	return 5

def decodeRepeatIndicator(bv, validate=False):
	return int(bv[6:8])

def decodeUserID(bv, validate=False):
	return int(bv[8:38])

def decodeAISversion(bv, validate=False):
	return int(bv[38:40])

def decodeIMOnumber(bv, validate=False):
	return int(bv[40:70])

def decodecallsign(bv, validate=False):
	return aisstring.decode(bv[70:112])

def decodename(bv, validate=False):
	return aisstring.decode(bv[112:232])

def decodeshipandcargo(bv, validate=False):
	return int(bv[232:240])

def decodedimA(bv, validate=False):
	return int(bv[240:249])

def decodedimB(bv, validate=False):
	return int(bv[249:258])

def decodedimC(bv, validate=False):
	return int(bv[258:264])

def decodedimD(bv, validate=False):
	return int(bv[264:270])

def decodefixtype(bv, validate=False):
	return int(bv[270:274])

def decodeETA(bv, validate=False):
	return int(bv[274:294])

def decodedraught(bv, validate=False):
	return Decimal(int(bv[294:302]))/Decimal('10')

def decodedestination(bv, validate=False):
	return aisstring.decode(bv[302:422])

def decodedte(bv, validate=False):
	return int(bv[422:423])

def decodeSpare(bv, validate=False):
	return 0

def printFields(params, out=sys.stdout, format='std'):
	'''Print a shipdata message to stdout.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 5 (field automatically set to "5")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - AISversion(uint): Compliant with what edition.  0 is the first edition.
	  - IMOnumber(uint): vessel identification number (different than mmsi)
	  - callsign(aisstr6): Ship radio call sign
	  - name(aisstr6): Vessel name
	  - shipandcargo(uint): what
	  - dimA(uint): FIX: write a description
	  - dimB(uint): FIX: write a description
	  - dimC(uint): FIX: write a description
	  - dimD(uint): FIX: write a description
	  - fixtype(uint): Method used for positioning
	  - ETA(uint): FIX: break this out
	  - draught(udecimal): Maximum present static draught.  25.5 is 25.5 m or greater
	  - destination(aisstr6): Where is the vessel going
	  - dte(uint): Data terminal ready
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("shipdata:\n")
		out.write("	MessageID:        "+str(params['MessageID'])+"\n")
		out.write("	RepeatIndicator:  "+str(params['RepeatIndicator'])+"\n")
		out.write("	UserID:           "+str(params['UserID'])+"\n")
		out.write("	AISversion:       "+str(params['AISversion'])+"\n")
		out.write("	IMOnumber:        "+str(params['IMOnumber'])+"\n")
		out.write("	callsign:         "+str(params['callsign'])+"\n")
		out.write("	name:             "+str(params['name'])+"\n")
		out.write("	shipandcargo:     "+str(params['shipandcargo'])+"\n")
		out.write("	dimA:             "+str(params['dimA'])+"\n")
		out.write("	dimB:             "+str(params['dimB'])+"\n")
		out.write("	dimC:             "+str(params['dimC'])+"\n")
		out.write("	dimD:             "+str(params['dimD'])+"\n")
		out.write("	fixtype:          "+str(params['fixtype'])+"\n")
		out.write("	ETA:              "+str(params['ETA'])+"\n")
		out.write("	draught:          "+str(params['draught'])+"\n")
		out.write("	destination:      "+str(params['destination'])+"\n")
		out.write("	dte:              "+str(params['dte'])+"\n")
		out.write("	Spare:            "+str(params['Spare'])+"\n")

	return # Nothing to return



######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['MessageID'] = 5
	params['RepeatIndicator'] = 1
	params['UserID'] = 1193046
	params['AISversion'] = 0
	params['IMOnumber'] = 3210
	params['callsign'] = 'PIRATE1'
	params['name'] = 'BLACK PEARL@@@@@@@@@'
	params['shipandcargo'] = 55
	params['dimA'] = 10
	params['dimB'] = 11
	params['dimC'] = 12
	params['dimD'] = 13
	params['fixtype'] = 1
	params['ETA'] = 0
	params['draught'] = Decimal('21.1')
	params['destination'] = 'NOWHERE@@@@@@@@@@@@@'
	params['dte'] = 0
	params['Spare'] = 0

	return params

class Testshipdata(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the shipdata message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['MessageID'],params['MessageID'])
		self.failUnlessEqual(r['RepeatIndicator'],params['RepeatIndicator'])
		self.failUnlessEqual(r['UserID'],params['UserID'])
		self.failUnlessEqual(r['AISversion'],params['AISversion'])
		self.failUnlessEqual(r['IMOnumber'],params['IMOnumber'])
		self.failUnlessEqual(r['callsign'],params['callsign'])
		self.failUnlessEqual(r['name'],params['name'])
		self.failUnlessEqual(r['shipandcargo'],params['shipandcargo'])
		self.failUnlessEqual(r['dimA'],params['dimA'])
		self.failUnlessEqual(r['dimB'],params['dimB'])
		self.failUnlessEqual(r['dimC'],params['dimC'])
		self.failUnlessEqual(r['dimD'],params['dimD'])
		self.failUnlessEqual(r['fixtype'],params['fixtype'])
		self.failUnlessEqual(r['ETA'],params['ETA'])
		self.failUnlessAlmostEqual(r['draught'],params['draught'],7)
		self.failUnlessEqual(r['destination'],params['destination'])
		self.failUnlessEqual(r['dte'],params['dte'])
		self.failUnlessEqual(r['Spare'],params['Spare'])

############################################################
if __name__=='__main__':

    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)

    #sys.exit('EARLY EXIT - FIX: remove')
    myparser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
                        help='run the unit tests')
    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                        help='Make the test output verbose')


    myparser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
                        help='decode a "shipdata" AIS message')
    myparser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
                        help='encode a "shipdata" AIS message')

		 
    # FIX: remove nmea from binary messages.  No way to build the whole packet?
    # FIX: or build the surrounding msg 8 for a broadcast?
    typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
    myparser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
			,default='nmeapayload'
			,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')

    myparser.add_option('--RepeatIndicator-field', dest='RepeatIndicatorField',default=0
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--UserID-field', dest='UserIDField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--AISversion-field', dest='AISversionField',default=0
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--IMOnumber-field', dest='IMOnumberField',default=0
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--callsign-field', dest='callsignField',default='@@@@@@@'
	,help='Field parameter value [default: %default]'
	,metavar='aisstr6'
	,type='string'
	)
    myparser.add_option('--name-field', dest='nameField',default='@@@@@@@@@@@@@@@@@@@@'
	,help='Field parameter value [default: %default]'
	,metavar='aisstr6'
	,type='string'
	)
    myparser.add_option('--shipandcargo-field', dest='shipandcargoField',default=0
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--dimA-field', dest='dimAField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--dimB-field', dest='dimBField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--dimC-field', dest='dimCField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--dimD-field', dest='dimDField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--fixtype-field', dest='fixtypeField',default=0
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--ETA-field', dest='ETAField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)
    myparser.add_option('--draught-field', dest='draughtField',default=Decimal('0')
	,help='Field parameter value [default: %default]'
	,metavar='udecimal'
	,type='string'
	)
    myparser.add_option('--destination-field', dest='destinationField',default='@@@@@@@@@@@@@@@@@@@@'
	,help='Field parameter value [default: %default]'
	,metavar='aisstr6'
	,type='string'
	)
    myparser.add_option('--dte-field', dest='dteField'
	,help='Field parameter value [default: %default]'
	,metavar='uint'
	,type='int'
	)

    (options,args) = myparser.parse_args()
    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	sys.argv= [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc

    if options.unittest:
	sys.argv = [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	unittest.main()

    if options.doEncode:
		# First make sure all non required options are specified
		if None==options.RepeatIndicatorField: myparser.error("missing value for RepeatIndicatorField")
		if None==options.UserIDField: myparser.error("missing value for UserIDField")
		if None==options.AISversionField: myparser.error("missing value for AISversionField")
		if None==options.IMOnumberField: myparser.error("missing value for IMOnumberField")
		if None==options.callsignField: myparser.error("missing value for callsignField")
		if None==options.nameField: myparser.error("missing value for nameField")
		if None==options.shipandcargoField: myparser.error("missing value for shipandcargoField")
		if None==options.dimAField: myparser.error("missing value for dimAField")
		if None==options.dimBField: myparser.error("missing value for dimBField")
		if None==options.dimCField: myparser.error("missing value for dimCField")
		if None==options.dimDField: myparser.error("missing value for dimDField")
		if None==options.fixtypeField: myparser.error("missing value for fixtypeField")
		if None==options.ETAField: myparser.error("missing value for ETAField")
		if None==options.draughtField: myparser.error("missing value for draughtField")
		if None==options.destinationField: myparser.error("missing value for destinationField")
		if None==options.dteField: myparser.error("missing value for dteField")
		msgDict={
			'MessageID': '5',
			'RepeatIndicator': options.RepeatIndicatorField,
			'UserID': options.UserIDField,
			'AISversion': options.AISversionField,
			'IMOnumber': options.IMOnumberField,
			'callsign': options.callsignField,
			'name': options.nameField,
			'shipandcargo': options.shipandcargoField,
			'dimA': options.dimAField,
			'dimB': options.dimBField,
			'dimC': options.dimCField,
			'dimD': options.dimDField,
			'fixtype': options.fixtypeField,
			'ETA': options.ETAField,
			'draught': options.draughtField,
			'destination': options.destinationField,
			'dte': options.dteField,
			'Spare': '0',
		}
		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')

    if options.doDecode:
		for msg in args:
			if 'binary'==options.ioType:
			    bv = BitVector(bitstring=msg)
			    printFields(decode(bv))
			elif 'nmeapayload'==options.ioType:
			    bv = binary.ais6tobitvec(msg)
			    printFields(decode(bv))
			elif 'nmea'==options.ioType:
			    bv = binary.ais6tobitvec(msg.split(',')[5])
			    printFields(decode(bv))
			else: sys.exit('ERROR: unknown ioType.  Help!')

