#!/usr/bin/env python

__version__ = '$Revision: 4799 $'.split()[1]
__date__ = '$Date: 2006-09-25 11:09:02 -0400 (Mon, 25 Sep 2006) $'.split()[1]
__author__ = 'John Doe FIX: put in the authors name'

__doc__='''
Handle creation and extraction of NMEA strings.  Maybe need a separate VDM class like ais-py?

@requires: U{lxml<http://codespeak.net/lxml/>} - For libxml2 ElementTree interface.  Not actually required for the template, but this is just a demo or requirements.
@requires: U{Python<http://python.org/>} >= 2.4
@requires: U{epydoc<http://epydoc.sourceforge.net/>} >= 3.0alpha3
@requires: BitVector

@author: U{'''+__author__+'''<http://schwehr.org/>}
@version: ''' + __version__ +'''
@copyright: 2006
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ myparser
@since: 2006-Sep-26  FIX: replace with the file creation date
@status: under development
@organization: U{CCOM<http://ccom.unh.edu/>} - FIX: if not CCOM change the name and link

@license: GPL v2
'''

# Python standard libraries
import time, sys

# Local Modules
import binary

import re


def checksumStr(data):
    """
    Take a NMEA 0183 string and compute the checksum.
    @param data: NMEA message.  Leading ?/! and training checksum are optional
    @type data: str
    @return: hexidecimal value
    @rtype: str

    Checksum is calculated by xor'ing everything between ? or ! and the *

    >>> checksumStr("!AIVDM,1,1,,B,35MsUdPOh8JwI:0HUwquiIFH21>i,0*09")
    '09'
    >>> checksumStr("AIVDM,1,1,,B,35MsUdPOh8JwI:0HUwquiIFH21>i,0")
    '09'
    """

    # FIX: strip off new line at the end too
    if data[0]=='!' or data[0]=='?': data = data[1:]
    if data[-1]=='*': data = data[:-1]
    if data[-3]=='*': data = data[:-3]
    # FIX: rename sum to not shadown builting function
    sum=0
    for c in data: sum = sum ^ ord(c)
    sumHex = "%x" % sum
    if len(sumHex)==1: sumHex = '0'+sumHex
    return sumHex.upper()


######################################################################
# common variables
nmeaChecksumRegExStr = r"""\,[0-9]\*[0-9A-F][0-9A-F]"""
nmeaChecksumRE = re.compile(nmeaChecksumRegExStr)

def isChecksumValid(nmeaStr, allowTailData=True):
    """Return True if the string checks out with the checksum

    @param allowTailData: Permit handing of Coast Guard format with data after the checksum
    @param data: NMEA message.  Leading ?/! are optional
    @type data: str
    @return: True if the checksum matches
    @rtype: bool

    >>> isChecksumValid("!AIVDM,1,1,,B,35MsUdPOh8JwI:0HUwquiIFH21>i,0*09")
    True

    Corrupted:

    >>> isChecksumValid("!AIVDM,11,1,,B,35MsUdPOh8JwI:0HUwquiIFH21>i,0*09")
    False
    """

    if allowTailData:
	match = nmeaChecksumRE.search(nmeaStr)
	if not match: return False
	nmeaStr = nmeaStr[:match.end()]
	#if checksum.upper()==checksumStr(nmeaStr[match.end()


    if nmeaStr[-3]!='*': 
	print 'FIX: warning... bad nmea string'
	return False  # Bad string without proper checksum
    checksum=nmeaStr[-2:]
    if checksum.upper()==checksumStr(nmeaStr).upper(): return True
    return False

######################################################################
if __name__=='__main__':
    from optparse import OptionParser
    myparser = OptionParser(usage="%prog [options]",
			    version="%prog "+__version__)
    myparser.add_option('--test','--doc-test',dest='doctest',default=False,action='store_true',
                        help='run the documentation tests')
    myparser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
                        help='run the tests run in verbose mode')
    (options,args) = myparser.parse_args()

    success=True

    if options.doctest:
	import os; print os.path.basename(sys.argv[0]), 'doctests ...',
	argvOrig = sys.argv
	sys.argv= [sys.argv[0]]
	if options.verbose: sys.argv.append('-v')
	import doctest
	numfail,numtests=doctest.testmod()
	if numfail==0: print 'ok'
	else: 
	    print 'FAILED'
	    success=False
	sys.argv = argvOrig # Restore the original args
	del argvOrig # hide from epydoc

    if not success:
	sys.exit('Something Failed')

    del success # Hide success from epydoc
